import '../types/mapping.dart';

/// Database-specific options used by drift.
///
/// Instances of this class are primarily meant to be generated by `drift_dev`,
/// which will override the database options based on compile-time options.
class DriftDatabaseOptions {
  /// Configuration for [SqlTypes] describing how to map Dart values from and to
  /// SQL values.
  final SqlTypes types;

  /// Creates database-specific database options.
  ///
  /// When [storeDateTimeAsText] is enabled (it defaults to `false` for
  /// backwards-compatibility), drift's datetime columns will be stored as text.
  /// By default, they will be stored as unix timestamps (integers).
  ///
  /// For details on how datetimes can be stored, see [the documentation].
  ///
  /// [the documentation]: https://drift.simonbinder.eu/docs/getting-started/advanced_dart_tables/#supported-column-types
  const DriftDatabaseOptions({
    bool storeDateTimeAsText = false,
  }) : types =
            storeDateTimeAsText ? const SqlTypes(true) : const SqlTypes(false);
}
